@@ -19,7 +19,13 @@
 #include <fcntl.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/prctl.h>
+#ifdef _WIN32
+  #include <wtypes.h>
+  #include <winbase.h>
+#else
+  #include <sys/prctl.h>
+  #include <sys/wait.h>
+#endif
 #include <ctype.h>
 #include <libavcodec/avcodec.h>
 #include <libswscale/swscale.h>
@@ -50,6 +56,7 @@
 #include "gui.h"
 #include "logging.h"
 #include "../stringutils.h"
+#include "../compat.h"
 
 struct viddata
 {
@@ -66,10 +73,10 @@ struct viddata
   int audiopipe;
   SwrContext* swrctx;
 #endif
-  GtkTextBuffer* buffer; // TODO: struct buffer array, for PMs
+  GtkTextBuffer* buffer;
   GtkAdjustment* scroll;
-  GtkBuilder* gui;
 };
+struct viddata* data;
 
 int tc_client[2];
 int tc_client_in[2];
@@ -78,6 +85,9 @@ const char* mycolor=0;
 char* nickname=0;
 char frombuild=0; // Running from the build directory
 #define TC_CLIENT (frombuild?"./tc_client":"tc_client")
+#ifdef _WIN32
+  PROCESS_INFORMATION coreprocess={.hProcess=0};
+#endif
 
 void updatescaling(struct viddata* data, unsigned int width, unsigned int height)
 {
@@ -115,48 +125,75 @@ void updatescaling(struct viddata* data, unsigned int width, unsigned int height
   }
 }
 
-void printchat(struct viddata* data, const char* text)
+void printchat(const char* text, const char* pm)
 {
-  char bottom=autoscroll_before(data->scroll);
+  GtkAdjustment* scroll;
+  GtkTextBuffer* buffer;
+  struct user* user;
+  if(pm && (user=finduser(pm)))
+  {
+    pm_open(pm, 0);
+    scroll=user->pm_scroll;
+    buffer=user->pm_buffer;
+  }else{
+    scroll=data->scroll;
+    buffer=data->buffer;
+  }
+  char bottom=autoscroll_before(scroll);
   // Insert new content
   GtkTextIter end;
-  gtk_text_buffer_get_end_iter(data->buffer, &end);
-  gtk_text_buffer_insert(data->buffer, &end, "\n", -1);
-  gtk_text_buffer_insert(data->buffer, &end, text, -1);
-  if(bottom){autoscroll_after(data->scroll);}
+  gtk_text_buffer_get_end_iter(buffer, &end);
+  gtk_text_buffer_insert(buffer, &end, "\n", -1);
+  gtk_text_buffer_insert(buffer, &end, text, -1);
+  if(bottom){autoscroll_after(scroll);}
 }
 
-void printchat_color(struct viddata* data, const char* text, const char* color, unsigned int offset)
+void printchat_color(const char* text, const char* color, unsigned int offset, const char* pm)
 {
-  char bottom=autoscroll_before(data->scroll);
+  GtkAdjustment* scroll;
+  GtkTextBuffer* buffer;
+  struct user* user;
+  if(pm && (user=finduser(pm)))
+  {
+    pm_open(pm, 0);
+    scroll=user->pm_scroll;
+    buffer=user->pm_buffer;
+  }else{
+    scroll=data->scroll;
+    buffer=data->buffer;
+  }
+  char bottom=autoscroll_before(scroll);
   // Insert new content
   GtkTextIter end;
-  gtk_text_buffer_get_end_iter(data->buffer, &end);
-  gtk_text_buffer_insert(data->buffer, &end, "\n", -1);
+  gtk_text_buffer_get_end_iter(buffer, &end);
+  gtk_text_buffer_insert(buffer, &end, "\n", -1);
   int startnum=gtk_text_iter_get_offset(&end);
-  gtk_text_buffer_insert(data->buffer, &end, text, -1);
+  gtk_text_buffer_insert(buffer, &end, text, -1);
   // Set color if there was one
-  GtkTextIter start;
-  gtk_text_buffer_get_iter_at_offset(data->buffer, &start, startnum+offset);
-  gtk_text_buffer_apply_tag_by_name(data->buffer, color, &start, &end);
-  if(bottom){autoscroll_after(data->scroll);}
+  if(color)
+  {
+    GtkTextIter start;
+    gtk_text_buffer_get_iter_at_offset(buffer, &start, startnum+offset);
+    gtk_text_buffer_apply_tag_by_name(buffer, color, &start, &end);
+  }
+  if(bottom){autoscroll_after(scroll);}
 }
 
 char buf[1024];
 gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer datap)
 {
-  int fd=g_io_channel_unix_get_fd(iochannel);
-  struct viddata* data=datap;
+  gsize r;
   unsigned int i;
   for(i=0; i<1023; ++i)
   {
-    if(read(fd, &buf[i], 1)<1){printf("No more data\n"); gtk_main_quit(); return 0;}
+    g_io_channel_read_chars(iochannel, &buf[i], 1, &r, 0);
+    if(r<1){printf("No more data\n"); gtk_main_quit(); return 0;}
     if(buf[i]=='\r'||buf[i]=='\n'){break;}
   }
   buf[i]=0;
   if(!strncmp(buf, "Currently online: ", 18))
   {
-    printchat(data, buf);
+    printchat(buf, 0);
     char* next=&buf[16];
     while(next)
     {
@@ -177,7 +214,7 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
   // Start streams once we're properly connected
   if(!strncmp(buf, "Currently on cam: ", 18))
   {
-    printchat(data, buf);
+    printchat(buf, 0);
     char* next=&buf[16];
     while(next)
     {
@@ -191,13 +228,13 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
   }
   if(!strcmp(buf, "Password required"))
   {
-    gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(data->gui, "main")));
-    gtk_widget_show_all(GTK_WIDGET(gtk_builder_get_object(data->gui, "channelpasswordwindow")));
+    gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(gui, "main")));
+    gtk_widget_show_all(GTK_WIDGET(gtk_builder_get_object(gui, "channelpasswordwindow")));
     return 1;
   }
   if(buf[0]=='/') // For the /help text
   {
-    printchat(data, buf);
+    printchat(buf, 0);
     return 1;
   }
   // Remove escape codes and pick up the text color while we're at it
@@ -214,51 +251,81 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
   // Timestamped events
   if(buf[0]=='['&&isdigit(buf[1])&&isdigit(buf[2])&&buf[3]==':'&&isdigit(buf[4])&&isdigit(buf[5])&&buf[6]==']'&&buf[7]==' ')
   {
+    char* pm=0;
     char* nick=&buf[8];
     space=strchr(nick, ' ');
     if(!space){return 1;}
     if(space[-1]==':')
     {
-// TODO: handle /msg (PMs)
-      if(config_get_bool("soundradio_cmd") && !fork())
+      if(config_get_bool("soundradio_cmd"))
       {
-        execlp("sh", "sh", "-c", config_get_str("soundcmd"), (char*)0);
-        _exit(0);
-      }
-      if(!strncmp(space, " /mbs youTube ", 14) && config_get_bool("youtuberadio_cmd") && !fork())
-      {
-// TODO: store the PID and make sure it's dead before starting a new video?
-// TODO: only play videos from mods?
-        char* id=&space[14];
-        char* offset=strchr(id, ' ');
-        if(!offset){_exit(1);}
-        offset[0]=0;
-        offset=&offset[1];
-        char* end=strchr(offset, ' '); // Ignore any additional arguments after the offset (the modbot utility includes the video title here)
-        if(end){end[0]=0;}
-        // Handle format string
-        const char* fmt=config_get_str("youtubecmd");
-        int len=strlen(fmt)+1;
-        len+=strcount(fmt, "%i")*(strlen(id)-2);
-        len+=strcount(fmt, "%t")*(strlen(id)-2);
-        char cmd[len];
-        cmd[0]=0;
-        while(fmt[0])
+#ifdef _WIN32
+        char* cmd=strdup(config_get_str("soundcmd"));
+        w32_runcmd(cmd);
+        free(cmd);
+#else
+        if(!fork())
         {
-          if(!strncmp(fmt, "%i", 2)){strcat(cmd, id); fmt=&fmt[2]; continue;}
-          if(!strncmp(fmt, "%t", 2)){strcat(cmd, offset); fmt=&fmt[2]; continue;}
-          for(len=0; fmt[len] && strncmp(&fmt[len], "%i", 2) && strncmp(&fmt[len], "%t", 2); ++len);
-          strncat(cmd, fmt, len);
-          fmt=&fmt[len];
+          execlp("sh", "sh", "-c", config_get_str("soundcmd"), (char*)0);
+          _exit(0);
+        }
+#endif
+      }
+      if(!strncmp(space, " /mbs youTube ", 14) && config_get_bool("youtuberadio_cmd"))
+      {
+#ifndef _WIN32
+        if(!fork())
+#endif
+        {
+// TODO: store the PID and make sure it's dead before starting a new video? and upon /mbc?
+// TODO: only play videos from mods?
+          char* id=&space[14];
+          char* offset=strchr(id, ' ');
+          if(!offset){_exit(1);}
+          offset[0]=0;
+          offset=&offset[1];
+          char* end=strchr(offset, ' '); // Ignore any additional arguments after the offset (the modbot utility includes the video title here)
+          if(end){end[0]=0;}
+          // Handle format string
+          const char* fmt=config_get_str("youtubecmd");
+          int len=strlen(fmt)+1;
+          len+=strcount(fmt, "%i")*(strlen(id)-2);
+          len+=strcount(fmt, "%t")*(strlen(id)-2);
+          char cmd[len];
+          cmd[0]=0;
+          while(fmt[0])
+          {
+            if(!strncmp(fmt, "%i", 2)){strcat(cmd, id); fmt=&fmt[2]; continue;}
+            if(!strncmp(fmt, "%t", 2)){strcat(cmd, offset); fmt=&fmt[2]; continue;}
+            for(len=0; fmt[len] && strncmp(&fmt[len], "%i", 2) && strncmp(&fmt[len], "%t", 2); ++len);
+            strncat(cmd, fmt, len);
+            fmt=&fmt[len];
+          }
+#ifdef _WIN32
+          w32_runcmd(cmd);
+#else
+          execlp("sh", "sh", "-c", cmd, (char*)0);
+          _exit(0);
+#endif
+        }
+      }
+      // Handle incoming PMs
+      else if(!strncmp(space, " /msg ", 6))
+      {
+        char* msg=strchr(&space[6], ' ');
+        if(msg)
+        {
+          memmove(space, msg, strlen(msg)+1);
+          char* end=strchr(nick, ':');
+          pm=strndup(nick, end-nick);
         }
-        execlp("sh", "sh", "-c", cmd, (char*)0);
-        _exit(0);
       }
     }
-// TODO: handle logging PMs
-    if(config_get_bool("enable_logging")){logger_write(buf, channel, 0);}
+    if(config_get_bool("enable_logging")){logger_write(buf, channel, pm);}
     // Insert new content
-    printchat_color(data, buf, color, 8);
+    printchat_color(buf, color, 8, pm);
+    pm_highlight(pm);
+    free(pm);
     if(space[-1]!=':') // Not a message
     {
       if(!strcmp(space, " entered the channel"))
@@ -296,14 +363,14 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
   }
   if(!strcmp(buf, "Changed color") || !strncmp(buf, "Current color: ", 15))
   {
-    printchat_color(data, buf, color, 0);
+    printchat_color(buf, color, 0, 0);
     free((void*)mycolor);
     mycolor=color;
     return 1;
   }
   if(!strncmp(buf, "Color ", 6))
   {
-    printchat_color(data, buf, color, 0);
+    printchat_color(buf, color, 0, 0);
   }
   free(color);
   if(space && !strcmp(space, " is a moderator."))
@@ -403,7 +470,7 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
     unsigned int size=strtoul(&sizestr[1], 0, 0);
     if(!size){return 1;}
     unsigned char frameinfo;
-    read(fd, &frameinfo, 1);
+    g_io_channel_read_chars(iochannel, (gchar*)&frameinfo, 1, 0, 0);
     --size; // For the byte we read above
     AVPacket pkt;
     av_init_packet(&pkt);
@@ -413,7 +480,8 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
     unsigned int pos=0;
     while(pos<size)
     {
-      pos+=read(fd, pkt.data+pos, size-pos);
+      g_io_channel_read_chars(iochannel, (gchar*)pkt.data+pos, size-pos, &r, 0);
+      pos+=r;
     }
 #if defined(HAVE_AVRESAMPLE) || defined(HAVE_SWRESAMPLE)
     // Find the camera representation for the given ID (for decoder context)
@@ -446,13 +514,14 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
   unsigned char databuf[size+4];
   pkt.data=databuf;
   unsigned char frameinfo;
-  read(fd, &frameinfo, 1);
+  g_io_channel_read_chars(iochannel, (gchar*)&frameinfo, 1, 0, 0);
 // printf("Frametype-frame: %x\n", ((unsigned int)frameinfo&0xf0)/16);
 // printf("Frametype-codec: %x\n", (unsigned int)frameinfo&0xf);
   unsigned int pos=0;
   while(pos<size)
   {
-    pos+=read(fd, pkt.data+pos, size-pos);
+    g_io_channel_read_chars(iochannel, (gchar*)pkt.data+pos, size-pos, &r, 0);
+    pos+=r;
   }
   if(!strcmp(&buf[7], "out"))
   {
@@ -606,7 +675,9 @@ gboolean handleresize(GtkWidget* widget, GdkEventConfigure* event, struct viddat
   {
     updatescaling(data, event->width, 0);
   }
+#ifndef _WIN32 // For some reason scrolling as a response to resizing freezes windows
   if(bottom){autoscroll_after(data->scroll);}
+#endif
   return 0;
 }
 
@@ -614,7 +685,9 @@ void handleresizepane(GObject* obj, GParamSpec* spec, struct viddata* data)
 {
   char bottom=autoscroll_before(data->scroll);
   updatescaling(data, 0, gtk_paned_get_position(GTK_PANED(obj)));
+#ifndef _WIN32
   if(bottom){autoscroll_after(data->scroll);}
+#endif
 }
 
 gboolean inputkeys(GtkWidget* widget, GdkEventKey* event, void* data)
@@ -672,6 +745,30 @@ gboolean inputkeys(GtkWidget* widget, GdkEventKey* event, void* data)
 void sendmessage(GtkEntry* entry, struct viddata* data)
 {
   const char* msg=gtk_entry_get_text(entry);
+  char* pm=0;
+  if(!strncmp(msg, "/pm ", 4))
+  {
+    pm_open(&msg[4], 1);
+    gtk_entry_set_text(entry, "");
+    return;
+  }
+  else if(msg[0]!='/') // If we're in a PM tab, send messages as PMs
+  {
+    GtkNotebook* tabs=GTK_NOTEBOOK(gtk_builder_get_object(gui, "tabs"));
+    int num=gtk_notebook_get_current_page(tabs);
+    if(num>0)
+    {
+      GtkWidget* page=gtk_notebook_get_nth_page(tabs, num);
+      struct user* user=user_find_by_tab(page);
+      if(!user) // Person we were PMing with left
+      {
+        gtk_entry_set_text(entry, "");
+        return;
+      }
+      pm=strdup(user->nick);
+      dprintf(tc_client_in[1], "/msg %s ", pm);
+    }
+  }
   dprintf(tc_client_in[1], "%s\n", msg);
   // Don't print commands
   if(!strcmp(msg, "/help") ||
@@ -696,26 +793,101 @@ void sendmessage(GtkEntry* entry, struct viddata* data)
     gtk_entry_set_text(entry, "");
     return;
   }
+  if(!strncmp(msg, "/msg ", 5))
+  {
+    const char* end=strchr(&msg[5], ' ');
+    if(end)
+    {
+      pm=strndup(&msg[5], end-&msg[5]);
+      struct user* user=finduser(pm);
+      if(!user)
+      {
+        gtk_entry_set_text(entry, "");
+        printchat("No such user", 0);
+        free(pm);
+        return;
+      }
+    }
+  }
   char text[strlen("[00:00] ")+strlen(nickname)+strlen(": ")+strlen(msg)+1];
   time_t timestamp=time(0);
   struct tm* t=localtime(&timestamp);
   sprintf(text, "[%02i:%02i] ", t->tm_hour, t->tm_min);
-  sprintf(&text[8], "%s: %s", nickname, msg);
-  if(config_get_bool("enable_logging")){logger_write(text, channel, 0);}
-  printchat_color(data, text, mycolor, 8);
+  if(pm && msg[0]=='/')
+  {
+    sprintf(&text[8], "%s: %s", nickname, &msg[6+strlen(pm)]);
+  }else{
+    sprintf(&text[8], "%s: %s", nickname, msg);
+  }
+  if(config_get_bool("enable_logging")){logger_write(text, channel, pm);}
+  printchat_color(text, mycolor, 8, pm);
   gtk_entry_set_text(entry, "");
+  free(pm);
 }
 
-void startsession(GtkButton* button, struct viddata* data)
+void startsession(GtkButton* button, void* x)
 {
-  gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(data->gui, "startwindow")));
-  gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(data->gui, "channelpasswordwindow")));
-  gtk_widget_show_all(GTK_WIDGET(gtk_builder_get_object(data->gui, "main")));
-  const char* nick=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(data->gui, "start_nick")));
-  channel=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(data->gui, "start_channel")));
-  const char* chanpass=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(data->gui, "channelpassword")));
-  const char* acc_user=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(data->gui, "acc_username")));
-  const char* acc_pass=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(data->gui, "acc_password")));
+  if(x!=(void*)-1)
+  {
+    // Populate the quick-connect fields with saved data
+    int channel_id=(int)(intptr_t)x;
+    char buf[256];
+    sprintf(buf, "channel%i_nick", channel_id);
+    gtk_entry_set_text(GTK_ENTRY(gtk_builder_get_object(gui, "cc_nick")), config_get_str(buf));
+
+    sprintf(buf, "channel%i_name", channel_id);
+    gtk_entry_set_text(GTK_ENTRY(gtk_builder_get_object(gui, "cc_channel")), config_get_str(buf));
+
+    sprintf(buf, "channel%i_password", channel_id);
+    gtk_entry_set_text(GTK_ENTRY(gtk_builder_get_object(gui, "cc_password")), config_get_str(buf));
+
+    sprintf(buf, "channel%i_accuser", channel_id);
+    gtk_entry_set_text(GTK_ENTRY(gtk_builder_get_object(gui, "acc_username")), config_get_str(buf));
+
+    sprintf(buf, "channel%i_accpass", channel_id);
+    gtk_entry_set_text(GTK_ENTRY(gtk_builder_get_object(gui, "acc_password")), config_get_str(buf));
+    // TODO: if account password is empty, open a password entry window similar to the channel password one
+  }
+  gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(gui, "startwindow")));
+  gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(gui, "channelconfig")));
+  gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(gui, "channelpasswordwindow")));
+  gtk_widget_show_all(GTK_WIDGET(gtk_builder_get_object(gui, "main")));
+  const char* nick=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(gui, "cc_nick")));
+  channel=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(gui, "cc_channel")));
+  const char* chanpass=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(gui, "channelpassword")));
+  if(!chanpass[0]){chanpass=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(gui, "cc_password")));}
+  const char* acc_user=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(gui, "acc_username")));
+  const char* acc_pass=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(gui, "acc_password")));
+#ifdef _WIN32
+  HANDLE h_tc_client0, h_tc_client1;
+  CreatePipe(&h_tc_client0, &h_tc_client1, &sa, 0);
+  HANDLE h_tc_client_in0, h_tc_client_in1;
+  CreatePipe(&h_tc_client_in0, &h_tc_client_in1, &sa, 0);
+  tc_client[0]=_open_osfhandle(h_tc_client0, _O_RDONLY);
+  tc_client[1]=_open_osfhandle(h_tc_client1, _O_WRONLY);
+  tc_client_in[0]=_open_osfhandle(h_tc_client_in0, _O_RDONLY);
+  tc_client_in[1]=_open_osfhandle(h_tc_client_in1, _O_WRONLY);
+  STARTUPINFO startup;
+  GetStartupInfo(&startup);
+  startup.dwFlags|=STARTF_USESTDHANDLES;
+  startup.hStdInput=h_tc_client_in0;
+  startup.hStdOutput=h_tc_client1;
+  char cmd[strlen("./tc_client -u    0")+strlen(acc_user)+strlen(channel)+strlen(nick)+strlen(chanpass)];
+  strcpy(cmd, "./tc_client ");
+  if(acc_user[0])
+  {
+    strcat(cmd, "-u ");
+    strcat(cmd, acc_user);
+    strcat(cmd, " ");
+  }
+  strcat(cmd, channel);
+  strcat(cmd, " ");
+  strcat(cmd, nick);
+  strcat(cmd, " ");
+  strcat(cmd, chanpass);
+  strcat(cmd, " ");
+  CreateProcess(0, cmd, 0, 0, 1, DETACHED_PROCESS, 0, 0, &startup, &coreprocess);
+#else
   pipe(tc_client);
   pipe(tc_client_in);
   if(!fork())
@@ -732,72 +904,46 @@ void startsession(GtkButton* button, struct viddata* data)
       execlp(TC_CLIENT, TC_CLIENT, channel, nick, chanpass, (char*)0);
     }
   }
+#endif
   if(acc_user[0]){dprintf(tc_client_in[1], "%s\n", acc_pass);}
   write(tc_client_in[1], "/color\n", 7);
   GIOChannel* tcchannel=g_io_channel_unix_new(tc_client[0]);
   g_io_channel_set_encoding(tcchannel, 0, 0);
   g_io_add_watch(tcchannel, G_IO_IN, handledata, data);
-  // Remember, if asked to
-  char save=0;
-  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gtk_builder_get_object(data->gui, "start_rememberchan"))))
-  {
-    config_set("remember_nick", gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(data->gui, "start_nick"))));
-    config_set("remember_chan", gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(data->gui, "start_channel"))));
-    config_set("remember_chan_nick", "True");
-    save=1;
-  }
-  else if(config_get_bool("remember_chan_nick")) // Remove previously remembered info
-  {
-    config_set("remember_nick", "");
-    config_set("remember_chan", "");
-    config_set("remember_chan_nick", "False");
-    save=1;
-  }
-  // Same for account
-  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gtk_builder_get_object(data->gui, "start_rememberacc"))))
-  {
-    config_set("remember_username", gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(data->gui, "acc_username"))));
-    config_set("remember_password", gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(data->gui, "acc_password"))));
-    config_set("remember_acc", "True");
-    save=1;
-  }
-  else if(config_get_bool("remember_acc")) // Remove previously remembered info
-  {
-    config_set("remember_username", "");
-    config_set("remember_password", "");
-    config_set("remember_acc", "False");
-    save=1;
-  }
-  if(save){config_save();}
 }
 
 int main(int argc, char** argv)
 {
   if(!strncmp(argv[0], "./", 2)){frombuild=1;}
-  struct viddata data={0,0,0,0,0};
+  struct viddata datax={0,0,0,0,0};
+  data=&datax;
   avcodec_register_all();
-  data.vdecoder=avcodec_find_decoder(AV_CODEC_ID_FLV1);
-  data.adecoder=avcodec_find_decoder(AV_CODEC_ID_NELLYMOSER);
+  data->vdecoder=avcodec_find_decoder(AV_CODEC_ID_FLV1);
+  data->adecoder=avcodec_find_decoder(AV_CODEC_ID_NELLYMOSER);
+#ifndef _WIN32
   signal(SIGCHLD, SIG_IGN);
+#else
+  frombuild=1;
+#endif
 
 #if defined(HAVE_AVRESAMPLE) || defined(HAVE_SWRESAMPLE)
   #ifdef HAVE_AVRESAMPLE
-  data.resamplectx=avresample_alloc_context();
-  av_opt_set_int(data.resamplectx, "in_channel_layout", AV_CH_FRONT_CENTER, 0);
-  av_opt_set_int(data.resamplectx, "in_sample_fmt", AV_SAMPLE_FMT_FLT, 0);
+  data->resamplectx=avresample_alloc_context();
+  av_opt_set_int(data->resamplectx, "in_channel_layout", AV_CH_FRONT_CENTER, 0);
+  av_opt_set_int(data->resamplectx, "in_sample_fmt", AV_SAMPLE_FMT_FLT, 0);
   // TODO: any way to get the sample rate from the frame/decoder? cam->frame->sample_rate seems to be 0
-  av_opt_set_int(data.resamplectx, "in_sample_rate", 11025, 0);
-  av_opt_set_int(data.resamplectx, "out_channel_layout", AV_CH_FRONT_CENTER, 0);
-  av_opt_set_int(data.resamplectx, "out_sample_fmt", AV_SAMPLE_FMT_S16, 0);
-  av_opt_set_int(data.resamplectx, "out_sample_rate", 22050, 0);
-  avresample_open(data.resamplectx);
+  av_opt_set_int(data->resamplectx, "in_sample_rate", 11025, 0);
+  av_opt_set_int(data->resamplectx, "out_channel_layout", AV_CH_FRONT_CENTER, 0);
+  av_opt_set_int(data->resamplectx, "out_sample_fmt", AV_SAMPLE_FMT_S16, 0);
+  av_opt_set_int(data->resamplectx, "out_sample_rate", 22050, 0);
+  avresample_open(data->resamplectx);
   #else
-  data.swrctx=swr_alloc_set_opts(0, AV_CH_FRONT_CENTER, AV_SAMPLE_FMT_S16, 22050, AV_CH_FRONT_CENTER, AV_SAMPLE_FMT_FLT, 11025, 0, 0);
-  swr_init(data.swrctx);
+  data->swrctx=swr_alloc_set_opts(0, AV_CH_FRONT_CENTER, AV_SAMPLE_FMT_S16, 22050, AV_CH_FRONT_CENTER, AV_SAMPLE_FMT_FLT, 11025, 0, 0);
+  swr_init(data->swrctx);
   #endif
   int audiopipe[2];
   pipe(audiopipe);
-  data.audiopipe=audiopipe[1];
+  data->audiopipe=audiopipe[1];
   if(!fork())
   {
     prctl(PR_SET_PDEATHSIG, SIGHUP);
@@ -809,7 +955,6 @@ int main(int argc, char** argv)
 #endif
 
   gtk_init(&argc, &argv);
-  GtkBuilder* gui;
   if(frombuild)
   {
     gui=gtk_builder_new_from_file("gtkgui.glade");
@@ -817,12 +962,11 @@ int main(int argc, char** argv)
     gui=gtk_builder_new_from_file(PREFIX "/share/tc_client/gtkgui.glade");
   }
   gtk_builder_connect_signals(gui, 0);
-  data.gui=gui;
 
 #ifdef HAVE_V4L2
   GtkWidget* item=GTK_WIDGET(gtk_builder_get_object(gui, "menuitem_broadcast_camera"));
-  g_signal_connect(item, "toggled", G_CALLBACK(togglecam), &data);
-  data.vencoder=avcodec_find_encoder(AV_CODEC_ID_FLV1);
+  g_signal_connect(item, "toggled", G_CALLBACK(togglecam), data);
+  data->vencoder=avcodec_find_encoder(AV_CODEC_ID_FLV1);
 #else
   GtkWidget* item=GTK_WIDGET(gtk_builder_get_object(gui, "menuitem_broadcast"));
   gtk_widget_destroy(item);
@@ -831,36 +975,20 @@ int main(int argc, char** argv)
   item=GTK_WIDGET(gtk_builder_get_object(gui, "menuitem_options_settings"));
   g_signal_connect(item, "activate", G_CALLBACK(showsettings), gui);
   
-  data.box=GTK_WIDGET(gtk_builder_get_object(gui, "cambox"));
+  data->box=GTK_WIDGET(gtk_builder_get_object(gui, "cambox"));
   userlistwidget=GTK_WIDGET(gtk_builder_get_object(gui, "userlistbox"));
   GtkWidget* chatview=GTK_WIDGET(gtk_builder_get_object(gui, "chatview"));
-  data.scroll=gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(gtk_builder_get_object(gui, "chatscroll")));
+  data->scroll=gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(gtk_builder_get_object(gui, "chatscroll")));
 
-  data.buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(chatview));
-  #define colormap(code, color) gtk_text_buffer_create_tag(data.buffer, code, "foreground", color, (char*)0)
-  colormap("[31", "#821615");
-  colormap("[31;1", "#c53332");
-  colormap("[33", "#a08f23");
-  //colormap("[33", "#a78901");
-  colormap("[33;1", "#919104");
-  colormap("[32;1", "#7bb224");
-  //colormap("[32;1", "#7db257");
-  colormap("[32", "#487d21");
-  colormap("[36", "#00a990");
-  colormap("[34;1", "#32a5d9");
-  //colormap("[34;1", "#1d82eb");
-  colormap("[34", "#1965b6");
-  colormap("[35", "#5c1a7a");
-  colormap("[35;1", "#9d5bb5");
-  //colormap("[35;1", "#c356a3");
-  //colormap("[35;1", "#b9807f");
+  data->buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(chatview));
+  buffer_setup_colors(data->buffer);
 
   GtkWidget* panes=GTK_WIDGET(gtk_builder_get_object(gui, "vpaned"));
-  g_signal_connect(panes, "notify::position", G_CALLBACK(handleresizepane), &data);
+  g_signal_connect(panes, "notify::position", G_CALLBACK(handleresizepane), data);
 
   GtkWidget* inputfield=GTK_WIDGET(gtk_builder_get_object(gui, "inputfield"));
-  g_signal_connect(inputfield, "activate", G_CALLBACK(sendmessage), &data);
-  g_signal_connect(inputfield, "key-press-event", G_CALLBACK(inputkeys), &data);
+  g_signal_connect(inputfield, "activate", G_CALLBACK(sendmessage), data);
+  g_signal_connect(inputfield, "key-press-event", G_CALLBACK(inputkeys), data);
 
   config_load();
   // Sound
@@ -876,39 +1004,69 @@ int main(int argc, char** argv)
   g_signal_connect(option, "toggled", G_CALLBACK(toggle_youtubecmd), gui);
 
   GtkWidget* window=GTK_WIDGET(gtk_builder_get_object(gui, "main"));
-  g_signal_connect(window, "configure-event", G_CALLBACK(handleresize), &data);
+  g_signal_connect(window, "configure-event", G_CALLBACK(handleresize), data);
 
   // Start window and channel password window signals
-  GtkWidget* button=GTK_WIDGET(gtk_builder_get_object(gui, "connectbutton"));
-  g_signal_connect(button, "clicked", G_CALLBACK(startsession), &data);
-  button=GTK_WIDGET(gtk_builder_get_object(gui, "channelpasswordbutton"));
-  g_signal_connect(button, "clicked", G_CALLBACK(startsession), &data);
+  GtkWidget* button=GTK_WIDGET(gtk_builder_get_object(gui, "channelpasswordbutton"));
+  g_signal_connect(button, "clicked", G_CALLBACK(startsession), (void*)-1); // &data);
   button=GTK_WIDGET(gtk_builder_get_object(gui, "channelpassword"));
-  g_signal_connect(button, "activate", G_CALLBACK(startsession), &data);
+  g_signal_connect(button, "activate", G_CALLBACK(startsession), (void*)-1); // &data);
   GtkWidget* startwindow=GTK_WIDGET(gtk_builder_get_object(gui, "startwindow"));
-  // Set channel and nick from last session
-  if(config_get_bool("remember_chan_nick"))
+  // Connect signal for quick connect
+  item=GTK_WIDGET(gtk_builder_get_object(gui, "start_menu_connect"));
+  struct channelopts cc_connect={-1,0};
+  g_signal_connect(item, "activate", G_CALLBACK(channeldialog), &cc_connect);
+  // Connect signal for the add option
+  item=GTK_WIDGET(gtk_builder_get_object(gui, "start_menu_add"));
+  struct channelopts cc_add={-1,1};
+  g_signal_connect(item, "activate", G_CALLBACK(channeldialog), &cc_add);
+  // Connect signal for tab changing (to un-highlight)
+  item=GTK_WIDGET(gtk_builder_get_object(gui, "tabs"));
+  g_signal_connect(item, "switch-page", G_CALLBACK(pm_select), 0);
+  // Populate saved channels
+  GtkWidget* startbox=GTK_WIDGET(gtk_builder_get_object(gui, "startbox"));
+  int channelcount=config_get_int("channelcount");
+  if(channelcount)
   {
-    gtk_entry_set_text(GTK_ENTRY(gtk_builder_get_object(gui, "start_nick")), config_get_str("remember_nick"));
-    gtk_entry_set_text(GTK_ENTRY(gtk_builder_get_object(gui, "start_channel")), config_get_str("remember_chan"));
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtk_builder_get_object(gui, "start_rememberchan")), 1);
+    gtk_widget_destroy(GTK_WIDGET(gtk_builder_get_object(gui, "channel_placeholder")));
   }
-  // Set username and password from last session
-  if(config_get_bool("remember_acc"))
+  char buf[256];
+  int i;
+  for(i=0; i<channelcount; ++i)
   {
-    gtk_entry_set_text(GTK_ENTRY(gtk_builder_get_object(gui, "acc_username")), config_get_str("remember_username"));
-    gtk_entry_set_text(GTK_ENTRY(gtk_builder_get_object(gui, "acc_password")), config_get_str("remember_password"));
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtk_builder_get_object(gui, "start_rememberacc")), 1);
+    GtkWidget* box=gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+    #ifdef GTK_STYLE_CLASS_LINKED
+      GtkStyleContext* style=gtk_widget_get_style_context(box);
+      gtk_style_context_add_class(style, GTK_STYLE_CLASS_LINKED);
+    #endif
+    sprintf(buf, "channel%i_name", i);
+    const char* name=config_get_str(buf);
+    GtkWidget* connectbutton=gtk_button_new_with_label(name);
+    g_signal_connect(connectbutton, "clicked", G_CALLBACK(startsession), (void*)(intptr_t)i);
+    gtk_box_pack_start(GTK_BOX(box), connectbutton, 1, 1, 0);
+    GtkWidget* cfgbutton=gtk_button_new_from_icon_name("gtk-preferences", GTK_ICON_SIZE_BUTTON);
+    struct channelopts* opts=malloc(sizeof(struct channelopts));
+    opts->channel_id=i;
+    opts->save=1;
+    g_signal_connect(cfgbutton, "clicked", G_CALLBACK(channeldialog), opts);
+    gtk_box_pack_start(GTK_BOX(box), cfgbutton, 0, 0, 0);
+    gtk_box_pack_start(GTK_BOX(startbox), box, 0, 0, 2);
   }
   gtk_widget_show_all(startwindow);
 
   gtk_main();
  
+#ifdef _WIN32
+  if(coreprocess.hProcess)
+  {
+    TerminateProcess(coreprocess.hProcess, 0);
+  }
+#endif
   camera_cleanup();
 #ifdef HAVE_AVRESAMPLE
-  avresample_free(&data.resamplectx);
+  avresample_free(&data->resamplectx);
 #elif defined(HAVE_SWRESAMPLE)
-  swr_free(&data.swrctx);
+  swr_free(&data->swrctx);
 #endif
   return 0;
 }
